@@ -4,34 +4,40 @@ usage: sl cci [<command>] [<args>...] [options]
 Manage, delete, order compute instances
 
 The available commands are:
-  network         Manage network settings
-  edit            Edit details of a CCI
+  cancel          Cancel a running CCI
   create          Order and create a CCI
                     (see `sl cci create-options` for choices)
-  manage          Manage active CCI
-  list            List CCI's on the account
+  create-options  Output available available options when creating a CCI
   detail          Output details about a CCI
   dns             DNS related actions to a CCI
-  cancel          Cancel a running CCI
-  create-options  Output available available options when creating a CCI
-  reload          Reload the OS on a CCI based on its current configuration
+  edit            Edit details of a CCI
+  list            List CCI's on the account
+  nic-edit        Edit NIC settings
+  pause           Pauses an active CCI
+  power-off       Powers off a running CCI
+  power-on        Boots up a CCI
   ready           Check if a CCI has finished provisioning
+  reboot          Reboots a running CCI
+  reload          Reload the OS on a CCI based on its current configuration
+  resume          Resumes a paused CCI
 
 For several commands, <identifier> will be asked for. This can be the id,
 hostname or the ip address for a CCI.
 """
 # :copyright: (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
-# :license: BSD, see LICENSE for more details.
+# :license: MIT, see LICENSE for more details.
 
 from os import linesep
 import os.path
 
-from SoftLayer import CCIManager
+from SoftLayer import CCIManager, SshKeyManager, DNSManager
+from SoftLayer.utils import lookup
 from SoftLayer.CLI import (
     CLIRunnable, Table, no_going_back, confirm, mb_to_gb, listing,
     FormattedItem)
 from SoftLayer.CLI.helpers import (
-    CLIAbort, ArgumentError, SequentialOutput, NestedDict, blank, resolve_id)
+    CLIAbort, ArgumentError, NestedDict, blank, resolve_id, KeyValueTable,
+    update_with_template_args, FALSE_VALUES, export_to_template)
 
 
 class ListCCIs(CLIRunnable):
@@ -52,16 +58,16 @@ Options:
                 Cores, memory, primary_ip, backend_ip
 
 Filters:
+  -c --cpu=CPU             Number of CPU cores
+  -D --domain=DOMAIN       Domain portion of the FQDN. example: example.com
+  -d DC, --datacenter=DC   datacenter shortname (sng01, dal05, ...)
+  -H --hostname=HOST       Host portion of the FQDN. example: server
+  -m --memory=MEMORY       Memory in mebibytes
+  -n MBPS, --network=MBPS  Network port speed in Mbps
   --hourly                 Show hourly instances
   --monthly                Show monthly instances
-  -H --hostname=HOST       Host portion of the FQDN. example: server
-  -D --domain=DOMAIN       Domain portion of the FQDN. example: example.com
-  -c --cpu=CPU             Number of CPU cores
-  -m --memory=MEMORY       Memory in mebibytes (n * 1024)
-  -d DC, --datacenter=DC   datacenter shortname (sng01, dal05, ...)
-  -n MBPS, --network=MBPS  Network port speed in Mbps
   --tags=ARG               Only show instances that have one of these tags.
-                           Comma-separated. (production,db)
+                             Comma-separated. (production,db)
 
 For more on filters see 'sl help filters'
 """
@@ -97,7 +103,7 @@ For more on filters see 'sl help filters'
             guest = NestedDict(guest)
             t.add_row([
                 guest['id'],
-                guest['datacenter']['name'] or blank(),
+                guest['datacenter']['name'],
                 guest['fullyQualifiedDomainName'],
                 guest['maxCpu'],
                 mb_to_gb(guest['maxMemory']),
@@ -125,8 +131,7 @@ Options:
     @staticmethod
     def execute(client, args):
         cci = CCIManager(client)
-
-        t = Table(['Name', 'Value'])
+        t = KeyValueTable(['Name', 'Value'])
         t.align['Name'] = 'r'
         t.align['Value'] = 'l'
 
@@ -136,26 +141,41 @@ Options:
 
         t.add_row(['id', result['id']])
         t.add_row(['hostname', result['fullyQualifiedDomainName']])
-        t.add_row(['status', result['status']['name']])
-        t.add_row(['state', result['powerState']['name']])
-        t.add_row(['datacenter', result['datacenter']['name'] or blank()])
+        t.add_row(['status', FormattedItem(
+            result['status']['keyName'] or blank(),
+            result['status']['name'] or blank()
+        )])
+        t.add_row(['state', FormattedItem(
+            lookup(result, 'powerState', 'keyName'),
+            lookup(result, 'powerState', 'name'),
+        )])
+        t.add_row(['datacenter', result['datacenter']['name']])
+        operating_system = lookup(result,
+                                  'operatingSystem',
+                                  'softwareLicense',
+                                  'softwareDescription')
+        t.add_row([
+            'os',
+            FormattedItem(
+                operating_system['version'] or blank(),
+                operating_system['name'] or blank()
+            )])
+        t.add_row(['os_version', operating_system['version'] or blank()])
         t.add_row(['cores', result['maxCpu']])
         t.add_row(['memory', mb_to_gb(result['maxMemory'])])
         t.add_row(['public_ip', result['primaryIpAddress'] or blank()])
         t.add_row(['private_ip', result['primaryBackendIpAddress'] or blank()])
-        t.add_row([
-            'os',
-            FormattedItem(
-                result['operatingSystem']['softwareLicense']
-                ['softwareDescription']['referenceCode'] or blank(),
-                result['operatingSystem']['softwareLicense']
-                ['softwareDescription']['name'] or blank()
-            )])
         t.add_row(['private_only', result['privateNetworkOnlyFlag']])
         t.add_row(['private_cpu', result['dedicatedAccountHostOnlyFlag']])
         t.add_row(['created', result['createDate']])
         t.add_row(['modified', result['modifyDate']])
 
+        vlan_table = Table(['type', 'number', 'id'])
+        for vlan in result['networkVlans']:
+            vlan_table.add_row([
+                vlan['networkSpace'], vlan['vlanNumber'], vlan['id']])
+        t.add_row(['vlans', vlan_table])
+
         if result.get('notes'):
             t.add_row(['notes', result['notes']])
 
@@ -163,11 +183,10 @@ Options:
             t.add_row(['price rate', result['billingItem']['recurringFee']])
 
         if args.get('--passwords'):
-            user_strs = []
+            pass_table = Table(['username', 'password'])
             for item in result['operatingSystem']['passwords']:
-                user_strs.append(
-                    "%s %s" % (item['username'], item['password']))
-            t.add_row(['users', listing(user_strs)])
+                pass_table.add_row([item['username'], item['password']])
+            t.add_row(['users', pass_table])
 
         tag_row = []
         for tag in result['tagReferences']:
@@ -176,12 +195,13 @@ Options:
         if tag_row:
             t.add_row(['tags', listing(tag_row, separator=',')])
 
-        ptr_domains = client['Virtual_Guest'].\
-            getReverseDomainRecords(id=cci_id)
+        if not result['privateNetworkOnlyFlag']:
+            ptr_domains = client['Virtual_Guest'].\
+                getReverseDomainRecords(id=cci_id)
 
-        for ptr_domain in ptr_domains:
-            for ptr in ptr_domain['resourceRecords']:
-                t.add_row(['ptr', ptr['data']])
+            for ptr_domain in ptr_domains:
+                for ptr in ptr_domain['resourceRecords']:
+                    t.add_row(['ptr', ptr['data']])
 
         return t
 
@@ -194,12 +214,12 @@ Output available available options when creating a CCI
 
 Options:
   --all         Show all options. default if no other option provided
-  --datacenter  Show datacenter options
   --cpu         Show CPU options
-  --nic         Show NIC speed options
+  --datacenter  Show datacenter options
   --disk        Show disk options
-  --os          Show operating system options
   --memory      Show memory size options
+  --nic         Show NIC speed options
+  --os          Show operating system options
 """
     action = 'create-options'
     options = ['datacenter', 'cpu', 'nic', 'disk', 'os', 'memory']
@@ -218,7 +238,7 @@ Options:
         if args['--all']:
             show_all = True
 
-        t = Table(['Name', 'Value'])
+        t = KeyValueTable(['Name', 'Value'])
         t.align['Name'] = 'r'
         t.align['Value'] = 'l'
 
@@ -317,61 +337,236 @@ Options:
 
 class CreateCCI(CLIRunnable):
     """
-usage: sl cci create --hostname=HOST --domain=DOMAIN --cpu=CPU --memory=MEMORY
-                     (--os=OS | --image=GUID) (--hourly | --monthly) [options]
+usage: sl cci create [--key=KEY...] [options]
 
 Order/create a CCI. See 'sl cci create-options' for valid options
 
 Required:
-  -H --hostname=HOST  Host portion of the FQDN. example: server
-  -D --domain=DOMAIN  Domain portion of the FQDN example: example.com
-  -c --cpu=CPU        Number of CPU cores
-  -m --memory=MEMORY  Memory in mebibytes (n * 1024)
-
-  -o OS, --os=OS      OS install code. Tip: you can specify <OS>_LATEST
-  --image=GUID        Image GUID. See: 'sl image list' for reference
+  -c, --cpu=CPU        Number of CPU cores
+  -D, --domain=DOMAIN  Domain portion of the FQDN. example: example.com
+  -H, --hostname=HOST  Host portion of the FQDN. example: server
+  --image=GUID         Image GUID. See: 'sl image list' for reference
+  -m, --memory=MEMORY  Memory in mebibytes. example: 2048
+  -o, --os=OS          OS install code. Tip: you can specify <OS>_LATEST
 
   --hourly            Hourly rate instance type
   --monthly           Monthly rate instance type
 
 
 Optional:
-  -d DC, --datacenter=DC   datacenter shortname (sng01, dal05, ...)
-                           Note: Omitting this value defaults to the first
-                             available datacenter
-  -n MBPS, --network=MBPS  Network port speed in Mbps
-  --private                Allocate a private CCI
-  --dry-run, --test        Do not create CCI, just get a quote
-
-  -u --userdata=DATA       User defined metadata string
-  -F --userfile=FILE       Read userdata from file
-  -i --postinstall=URI     Post-install script to download
-                             (Only HTTPS executes, HTTP leaves file in /root)
-  --wait=SECONDS           Block until CCI is finished provisioning for up to X
-                             seconds before returning.
+  -d, --datacenter=DC    Datacenter shortname (sng01, dal05, ...)
+                         Note: Omitting this value defaults to the first
+                           available datacenter
+  --dedicated            Allocate a dedicated CCI (non-shared host)
+  --dry-run, --test      Do not create CCI, just get a quote
+  --export=FILE          Exports options to a template file
+  -F, --userfile=FILE    Read userdata from file
+                           (Only HTTPS executes, HTTP leaves file in /root)
+  -i, --postinstall=URI  Post-install script to download
+  -k, --key=KEY          SSH keys to add to the root user. Can be specified
+                           multiple times
+  --like=IDENTIFIER      Use the configuration from an existing CCI
+  -n, --network=MBPS     Network port speed in Mbps
+  --private              Forces the CCI to only have access the private
+                           network
+  -t, --template=FILE    A template file that defaults the command-line
+                           options using the long name in INI format
+  -u, --userdata=DATA    User defined metadata string
+  --vlan_public=VLAN     The ID of the public VLAN on which you want the CCI
+                           placed.
+  --vlan_private=VLAN    The ID of the private VLAN on which you want the CCI
+                           placed.
+  --wait=SECONDS         Block until CCI is finished provisioning for up to X
+                           seconds before returning
 """
     action = 'create'
     options = ['confirm']
+    required_params = ['--hostname', '--domain', '--cpu', '--memory']
 
-    @staticmethod
-    def execute(client, args):
+    @classmethod
+    def execute(cls, client, args):
+        update_with_template_args(args)
         cci = CCIManager(client)
+        cls._update_with_like_args(cci, args)
 
-        if args['--userdata'] and args['--userfile']:
+        # SSH keys may be a comma-separated list. Let's make it a real list.
+        if isinstance(args.get('--key'), str):
+            args['--key'] = args.get('--key').split(',')
+
+        cls._validate_args(args)
+
+        # Do not create CCI with --test or --export
+        do_create = not (args['--export'] or args['--test'])
+
+        t = Table(['Item', 'cost'])
+        t.align['Item'] = 'r'
+        t.align['cost'] = 'r'
+        data = cls._parse_create_args(client, args)
+
+        output = []
+        if args.get('--test'):
+            result = cci.verify_create_instance(**data)
+            total_monthly = 0.0
+            total_hourly = 0.0
+
+            t = Table(['Item', 'cost'])
+            t.align['Item'] = 'r'
+            t.align['cost'] = 'r'
+
+            for price in result['prices']:
+                total_monthly += float(price.get('recurringFee', 0.0))
+                total_hourly += float(price.get('hourlyRecurringFee', 0.0))
+                if args.get('--hourly'):
+                    rate = "%.2f" % float(price['hourlyRecurringFee'])
+                else:
+                    rate = "%.2f" % float(price['recurringFee'])
+
+                t.add_row([price['item']['description'], rate])
+
+            if args.get('--hourly'):
+                total = total_hourly
+            else:
+                total = total_monthly
+
+            billing_rate = 'monthly'
+            if args.get('--hourly'):
+                billing_rate = 'hourly'
+            t.add_row(['Total %s cost' % billing_rate, "%.2f" % total])
+            output.append(t)
+            output.append(FormattedItem(
+                None,
+                ' -- ! Prices reflected here are retail and do not '
+                'take account level discounts and are not guaranteed.')
+            )
+
+        if args['--export']:
+            export_file = args.pop('--export')
+            export_to_template(export_file, args, exclude=['--wait', '--test'])
+            return 'Successfully exported options to a template file.'
+
+        if do_create:
+            if args['--really'] or confirm(
+                    "This action will incur charges on your account. "
+                    "Continue?"):
+                result = cci.create_instance(**data)
+
+                t = KeyValueTable(['name', 'value'])
+                t.align['name'] = 'r'
+                t.align['value'] = 'l'
+                t.add_row(['id', result['id']])
+                t.add_row(['created', result['createDate']])
+                t.add_row(['guid', result['globalIdentifier']])
+                output.append(t)
+
+                if args.get('--wait'):
+                    ready = cci.wait_for_transaction(
+                        result['id'], int(args.get('--wait') or 1))
+                    t.add_row(['ready', ready])
+            else:
+                raise CLIAbort('Aborting CCI order.')
+
+        return output
+
+    @classmethod
+    def _validate_args(cls, args):
+        invalid_args = [k for k in cls.required_params if args.get(k) is None]
+        if invalid_args:
+            raise ArgumentError('Missing required options: %s'
+                                % ','.join(invalid_args))
+
+        if all([args['--userdata'], args['--userfile']]):
             raise ArgumentError('[-u | --userdata] not allowed with '
                                 '[-F | --userfile]')
+
+        if args['--hourly'] in FALSE_VALUES:
+            args['--hourly'] = False
+
+        if args['--monthly'] in FALSE_VALUES:
+            args['--monthly'] = False
+
+        if all([args['--hourly'], args['--monthly']]):
+            raise ArgumentError('[--hourly] not allowed with [--monthly]')
+
+        if not any([args['--hourly'], args['--monthly']]):
+            raise ArgumentError('One of [--hourly | --monthly] is required')
+
+        image_args = [args['--os'], args['--image']]
+        if all(image_args):
+            raise ArgumentError('[-o | --os] not allowed with [--image]')
+
+        if not any(image_args):
+            raise ArgumentError('One of [--os | --image] is required')
+
         if args['--userfile']:
             if not os.path.exists(args['--userfile']):
                 raise ArgumentError(
                     'File does not exist [-u | --userfile] = %s'
                     % args['--userfile'])
 
+    @staticmethod
+    def _update_with_like_args(cci, args):
+        """ Update arguments with options taken from a currently running CCI.
+
+        :param CCIManager args: A CCIManager
+        :param dict args: CLI arguments
+        """
+        if args['--like']:
+            cci_id = resolve_id(cci.resolve_ids, args.pop('--like'), 'CCI')
+            like_details = cci.get_instance(cci_id)
+            like_args = {
+                '--hostname': like_details['hostname'],
+                '--domain': like_details['domain'],
+                '--cpu': like_details['maxCpu'],
+                '--memory': like_details['maxMemory'],
+                '--hourly': like_details['hourlyBillingFlag'],
+                '--monthly': not like_details['hourlyBillingFlag'],
+                '--datacenter': like_details['datacenter']['name'],
+                '--network': like_details['networkComponents'][0]['maxSpeed'],
+                '--user-data': like_details['userData'] or None,
+                '--postinstall': like_details.get('postInstallScriptUri'),
+                '--dedicated': like_details['dedicatedAccountHostOnlyFlag'],
+                '--private': like_details['privateNetworkOnlyFlag'],
+            }
+
+            # Handle mutually exclusive options
+            like_image = lookup(like_details,
+                                'blockDeviceTemplateGroup',
+                                'globalIdentifier')
+            like_os = lookup(like_details,
+                             'operatingSystem',
+                             'softwareLicense',
+                             'softwareDescription',
+                             'referenceCode')
+            if like_image and not args.get('--os'):
+                like_args['--image'] = like_image
+            elif like_os and not args.get('--image'):
+                like_args['--os'] = like_os
+
+            if args.get('--hourly'):
+                like_args['--monthly'] = False
+
+            if args.get('--monthly'):
+                like_args['--hourly'] = False
+
+            # Merge like CCI options with the options passed in
+            for key, value in like_args.items():
+                if args.get(key) in [None, False]:
+                    args[key] = value
+
+    @staticmethod
+    def _parse_create_args(client, args):
+        """ Converts CLI arguments to arguments that can be passed into
+            CCIManager.create_instance.
+
+        :param dict args: CLI arguments
+        """
         data = {
             "hourly": args['--hourly'],
             "cpus": args['--cpu'],
             "domain": args['--domain'],
             "hostname": args['--hostname'],
             "private": args['--private'],
+            "dedicated": args['--dedicated'],
             "local_disk": True,
         }
 
@@ -416,61 +611,22 @@ Optional:
         if args.get('--postinstall'):
             data['post_uri'] = args.get('--postinstall')
 
-        t = Table(['Item', 'cost'])
-        t.align['Item'] = 'r'
-        t.align['cost'] = 'r'
+        # Get the SSH keys
+        if args.get('--key'):
+            keys = []
+            for key in args.get('--key'):
+                key_id = resolve_id(SshKeyManager(client).resolve_ids, key,
+                                    'SshKey')
+                keys.append(key_id)
+            data['ssh_keys'] = keys
 
-        if args.get('--test'):
-            result = cci.verify_create_instance(**data)
-            total_monthly = 0.0
-            total_hourly = 0.0
-            for price in result['prices']:
-                total_monthly += float(price.get('recurringFee', 0.0))
-                total_hourly += float(price.get('hourlyRecurringFee', 0.0))
-                if args.get('--hourly'):
-                    rate = "%.2f" % float(price['hourlyRecurringFee'])
-                else:
-                    rate = "%.2f" % float(price['recurringFee'])
+        if args.get('--vlan_public'):
+            data['public_vlan'] = args['--vlan_public']
 
-                t.add_row([price['item']['description'], rate])
+        if args.get('--vlan_private'):
+            data['private_vlan'] = args['--vlan_private']
 
-            if args.get('--hourly'):
-                total = total_hourly
-            else:
-                total = total_monthly
-
-            billing_rate = 'monthly'
-            if args.get('--hourly'):
-                billing_rate = 'hourly'
-            t.add_row(['Total %s cost' % billing_rate, "%.2f" % total])
-            output = SequentialOutput(blanks=False)
-            output.append(t)
-            output.append(FormattedItem(
-                '',
-                ' -- ! Prices reflected here are retail and do not '
-                'take account level discounts and are not guarenteed.')
-            )
-
-        elif args['--really'] or confirm(
-                "This action will incur charges on your account. Continue?"):
-            result = cci.create_instance(**data)
-
-            t = Table(['name', 'value'])
-            t.align['name'] = 'r'
-            t.align['value'] = 'l'
-            t.add_row(['id', result['id']])
-            t.add_row(['created', result['createDate']])
-            t.add_row(['guid', result['globalIdentifier']])
-            output = t
-        else:
-            raise CLIAbort('Aborting CCI order.')
-
-        if args.get('--wait') or 0 and not args.get('--test'):
-            ready = cci.wait_for_transaction(
-                result['id'], int(args.get('--wait') or 1))
-            t.add_row(['ready', ready])
-
-        return output
+        return data
 
 
 class ReadyCCI(CLIRunnable):
@@ -505,7 +661,7 @@ usage: sl cci reload <identifier> [options]
 Reload the OS on a CCI based on its current configuration
 
 Optional:
-    -i, --postinstall=URI   Post-install script to download
+    -i, --postinstall=URI  Post-install script to download
                              (Only HTTPS executes, HTTP leaves file in /root)
 """
 
@@ -542,145 +698,159 @@ Cancel a CCI
             CLIAbort('Aborted')
 
 
-class ManageCCI(CLIRunnable):
+class CCIPowerOff(CLIRunnable):
     """
-usage: sl cci manage poweroff <identifier> [--cycle | --soft] [options]
-       sl cci manage reboot <identifier> [--cycle | --soft] [options]
-       sl cci manage poweron <identifier> [options]
-       sl cci manage pause <identifier> [options]
-       sl cci manage resume <identifier> [options]
+usage: sl cci power-off <identifier> [--hard] [options]
 
-Manage active CCI
+Power off an active CCI
+
+Optional:
+    --hard  Perform a hard shutdown
 """
-    action = 'manage'
+    action = 'power-off'
     options = ['confirm']
 
     @classmethod
     def execute(cls, client, args):
-        if args['poweroff']:
-            return cls.exec_shutdown(client, args)
-
-        if args['reboot']:
-            return cls.exec_reboot(client, args)
-
-        if args['poweron']:
-            return cls.exec_poweron(client, args)
-
-        if args['pause']:
-            return cls.exec_pause(client, args)
-
-        if args['resume']:
-            return cls.exec_resume(client, args)
-
-    @staticmethod
-    def exec_shutdown(client, args):
         vg = client['Virtual_Guest']
         cci = CCIManager(client)
         cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        if args['--soft']:
-            result = vg.powerOffSoft(id=cci_id)
-        elif args['--cycle']:
-            result = vg.powerCycle(id=cci_id)
+        if args['--really'] or confirm('This will power off the CCI with id '
+                                       '%s. Continue?' % cci_id):
+            if args['--hard']:
+                vg.powerOff(id=cci_id)
+            else:
+                vg.powerOffSoft(id=cci_id)
         else:
-            result = vg.powerOff(id=cci_id)
-
-        return FormattedItem(result)
-
-    @staticmethod
-    def exec_poweron(client, args):
-        vg = client['Virtual_Guest']
-        cci = CCIManager(client)
-        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        return vg.powerOn(id=cci_id)
-
-    @staticmethod
-    def exec_pause(client, args):
-        vg = client['Virtual_Guest']
-        cci = CCIManager(client)
-        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        return vg.pause(id=cci_id)
-
-    @staticmethod
-    def exec_resume(client, args):
-        vg = client['Virtual_Guest']
-        cci = CCIManager(client)
-        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        return vg.resume(id=cci_id)
-
-    @staticmethod
-    def exec_reboot(client, args):
-        vg = client['Virtual_Guest']
-        cci = CCIManager(client)
-        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        if args['--cycle']:
-            result = vg.rebootHard(id=cci_id)
-        elif args['--soft']:
-            result = vg.rebootSoft(id=cci_id)
-        else:
-            result = vg.rebootDefault(id=cci_id)
-
-        return result
+            raise CLIAbort('Aborted.')
 
 
-class NetworkCCI(CLIRunnable):
+class CCIReboot(CLIRunnable):
     """
-usage: sl cci network port <identifier> --speed=SPEED (--public | --private)
-                           [options]
+usage: sl cci reboot <identifier> [--hard | --soft] [options]
 
-Manage network settings
+Reboot an active CCI
 
-Options:
-    --speed=SPEED  Port speed. 0 disables the port.
-                   [Options: 0, 10, 100, 1000, 10000]
-    --public       Public network
-    --private      Private network
+Optional:
+    --hard  Perform an abrupt reboot
+    --soft  Perform a graceful reboot
 """
-    action = 'network'
+    action = 'reboot'
+    options = ['confirm']
 
     @classmethod
     def execute(cls, client, args):
-        if args['port']:
-            return cls.exec_port(client, args)
+        vg = client['Virtual_Guest']
+        cci = CCIManager(client)
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
+        if args['--really'] or confirm('This will reboot the CCI with id '
+                                       '%s. Continue?' % cci_id):
+            if args['--hard']:
+                vg.rebootHard(id=cci_id)
+            elif args['--soft']:
+                vg.rebootSoft(id=cci_id)
+            else:
+                vg.rebootDefault(id=cci_id)
+        else:
+            raise CLIAbort('Aborted.')
 
-        if args['details']:
-            return cls.exec_detail(client, args)
 
-    @staticmethod
-    def exec_port(client, args):
-        public = True
-        if args['--private']:
-            public = False
+class CCIPowerOn(CLIRunnable):
+    """
+usage: sl cci power-on <identifier> [options]
+
+Power on a CCI
+"""
+    action = 'power-on'
+
+    @classmethod
+    def execute(cls, client, args):
+        vg = client['Virtual_Guest']
+        cci = CCIManager(client)
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
+        vg.powerOn(id=cci_id)
+
+
+class CCIPause(CLIRunnable):
+    """
+usage: sl cci pause <identifier> [options]
+
+Pauses an active CCI
+"""
+    action = 'pause'
+    options = ['confirm']
+
+    @classmethod
+    def execute(cls, client, args):
+        vg = client['Virtual_Guest']
+        cci = CCIManager(client)
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
+
+        if args['--really'] or confirm('This will pause the CCI with id '
+                                       '%s. Continue?' % cci_id):
+            vg.pause(id=cci_id)
+        else:
+            raise CLIAbort('Aborted.')
+
+
+class CCIResume(CLIRunnable):
+    """
+usage: sl cci resume <identifier> [options]
+
+Resumes a paused CCI
+"""
+    action = 'resume'
+
+    @classmethod
+    def execute(cls, client, args):
+        vg = client['Virtual_Guest']
+        cci = CCIManager(client)
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
+        vg.resume(id=cci_id)
+
+
+class NicEditCCI(CLIRunnable):
+    """
+usage: sl cci nic-edit <identifier> (public | private) --speed=SPEED [options]
+
+Manage NIC settings
+
+Options:
+    --speed=SPEED  Port speed. 0 disables the port.
+                     [Options: 0, 10, 100, 1000, 10000]
+"""
+    action = 'nic-edit'
+
+    @classmethod
+    def execute(cls, client, args):
+        public = args['public']
 
         cci = CCIManager(client)
         cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
 
-        result = cci.change_port_speed(cci_id, public, args['--speed'])
-        if result:
-            return "Success"
-        else:
-            return result
-
-    @staticmethod
-    def exec_detail(client, args):
-        # TODO this should print out default gateway and stuff
-        raise CLIAbort('Not implemented')
+        cci.change_port_speed(cci_id, public, args['--speed'])
 
 
 class CCIDNS(CLIRunnable):
     """
 usage: sl cci dns sync <identifier> [options]
 
-DNS related actions for a CCI
+Attempts to update DNS for the specified CCI. If you don't specify any
+arguments, it will attempt to update both the A and PTR records. If you don't
+want to update both records, you may use the -a or --ptr arguments to limit
+the records updated.
 
 Options:
-  -a, -A        Sync only the A record
-  --ptr, --PTR  Sync only the PTR record
+  -a         Sync the A record for the host
+  --ptr      Sync the PTR record for the host
+  --ttl=TTL  Sets the TTL for the A and/or PTR records
 """
     action = 'dns'
     options = ['confirm']
 
     @classmethod
     def execute(cls, client, args):
+        args['--ttl'] = args['--ttl'] or DNSManager.DEFAULT_TTL
         if args['sync']:
             return cls.dns_sync(client, args)
 
@@ -690,11 +860,14 @@ Options:
         dns = DNSManager(client)
         cci = CCIManager(client)
 
+        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
+        instance = cci.get_instance(cci_id)
+        zone_id = resolve_id(dns.resolve_ids, instance['domain'], name='zone')
+
         def sync_a_record():
-            #hostname = instance['fullyQualifiedDomainName']
-            records = dns.search_record(
-                instance['domain'],
-                instance['hostname'],
+            records = dns.get_records(
+                zone_id,
+                host=instance['hostname'],
             )
 
             if not records:
@@ -704,7 +877,7 @@ Options:
                     instance['hostname'],
                     'a',
                     instance['primaryIpAddress'],
-                    ttl=7200)
+                    ttl=args['--ttl'])
             else:
                 recs = filter(lambda x: x['type'].lower() == 'a', records)
                 if len(recs) != 1:
@@ -712,6 +885,7 @@ Options:
                                    "A record exists!" % len(recs))
                 rec = recs[0]
                 rec['data'] = instance['primaryIpAddress']
+                rec['ttl'] = args['--ttl']
                 dns.edit_record(rec)
 
         def sync_ptr_record():
@@ -721,6 +895,7 @@ Options:
             edit_ptr = None
             for ptr in ptr_domains['resourceRecords']:
                 if ptr['host'] == host_rec:
+                    ptr['ttl'] = args['--ttl']
                     edit_ptr = ptr
                     break
 
@@ -733,16 +908,13 @@ Options:
                     host_rec,
                     'ptr',
                     instance['fullyQualifiedDomainName'],
-                    ttl=7200)
-
-        cci_id = resolve_id(cci.resolve_ids, args.get('<identifier>'), 'CCI')
-        instance = cci.get_instance(cci_id)
+                    ttl=args['--ttl'])
 
         if not instance['primaryIpAddress']:
             raise CLIAbort('No primary IP address associated with this CCI')
 
         try:
-            zone = dns.get_zone(instance['domain'])
+            zone = dns.get_zone(zone_id)
         except DNSZoneNotFound:
             raise CLIAbort("Unable to create A record, "
                            "no zone found matching: %s" % instance['domain'])
@@ -755,13 +927,13 @@ Options:
             raise CLIAbort("Aborting DNS sync")
 
         both = False
-        if not args.get('--PTR') and not args.get('-A'):
+        if not args['--ptr'] and not args['-a']:
             both = True
 
-        if both or args.get('-A'):
+        if both or args['-a']:
             sync_a_record()
 
-        if both or args.get('--PTR'):
+        if both or args['--ptr']:
             sync_ptr_record()
 
 
@@ -772,10 +944,10 @@ usage: sl cci edit <identifier> [options]
 Edit CCI details
 
 Options:
-  -H --hostname=HOST  Host portion of the FQDN. example: server
   -D --domain=DOMAIN  Domain portion of the FQDN example: example.com
-  -u --userdata=DATA  User defined metadata string
   -F --userfile=FILE  Read userdata from file
+  -H --hostname=HOST  Host portion of the FQDN. example: server
+  -u --userdata=DATA  User defined metadata string
 """
     action = 'edit'
 
