@@ -4,7 +4,7 @@
     Hardware Manager/helpers
 
     :copyright: (c) 2013, SoftLayer Technologies, Inc. All rights reserved.
-    :license: BSD, see LICENSE for more details.
+    :license: MIT, see LICENSE for more details.
 """
 
 import socket
@@ -12,27 +12,33 @@ from SoftLayer.utils import NestedDict, query_filter, IdentifierMixin
 
 
 class HardwareManager(IdentifierMixin, object):
-    """ Manages hardware devices. """
+    """
+    Manages hardware devices.
+
+    :param SoftLayer.API.Client client: an API client instance
+    """
 
     def __init__(self, client):
-        """ HardwareManager initialization.
-
-        :param SoftLayer.API.Client client: an API client instance
-
-        """
+        #: A valid `SoftLayer.API.Client` object that will be used for all
+        #: actions.
         self.client = client
+        #: Reference to the SoftLayer_Hardware_Server API object.
         self.hardware = self.client['Hardware_Server']
+        #: Reference to the SoftLayer_Account API object.
         self.account = self.client['Account']
+        #: A list of resolver functions. Used primarily by the CLI to provide
+        #: a variety of methods for uniquely identifying an object such as
+        #: hostname and IP address.
         self.resolvers = [self._get_ids_from_ip, self._get_ids_from_hostname]
 
     def cancel_hardware(self, id, reason='unneeded', comment=''):
         """ Cancels the specified dedicated server.
 
         :param int id: The ID of the hardware to be cancelled.
-        :param bool immediate: If true, the hardware will be cancelled
-                               immediately. Otherwise, it will be
-                               scheduled to cancel on the anniversary date.
-        :param string reason: The reason code for the cancellation.
+        :param string reason: The reason code for the cancellation. This should
+                              come from :func:`get_cancellation_reasons`.
+        :param string comment: An optional comment to include with the
+                               cancellation.
         """
 
         reasons = self.get_cancellation_reasons()
@@ -75,7 +81,7 @@ class HardwareManager(IdentifierMixin, object):
     def list_hardware(self, tags=None, cpus=None, memory=None, hostname=None,
                       domain=None, datacenter=None, nic_speed=None,
                       public_ip=None, private_ip=None, **kwargs):
-        """ List all hardware.
+        """ List all hardware (servers and bare metal computing instances).
 
         :param list tags: filter based on tags
         :param integer cpus: filter based on number of CPUS
@@ -87,6 +93,9 @@ class HardwareManager(IdentifierMixin, object):
         :param string public_ip: filter based on public ip address
         :param string private_ip: filter based on private ip address
         :param dict \*\*kwargs: response-level arguments (limit, offset, etc.)
+        :returns: Returns a list of dictionaries representing the matching
+                  hardware. This list will contain both dedicated servers and
+                  bare metal computing instances
 
         """
         if 'mask' not in kwargs:
@@ -97,11 +106,11 @@ class HardwareManager(IdentifierMixin, object):
                 'hardwareStatusId',
                 'globalIdentifier',
                 'fullyQualifiedDomainName',
-                'processorCoreAmount',
+                'processorPhysicalCoreAmount',
                 'memoryCapacity',
                 'primaryBackendIpAddress',
                 'primaryIpAddress',
-                'datacenter.name',
+                'datacenter',
             ])
             kwargs['mask'] = "mask[%s]" % ','.join(items)
 
@@ -113,7 +122,8 @@ class HardwareManager(IdentifierMixin, object):
             }
 
         if cpus:
-            _filter['hardware']['processorCoreAmount'] = query_filter(cpus)
+            _filter['hardware']['processorPhysicalCoreAmount'] = \
+                query_filter(cpus)
 
         if memory:
             _filter['hardware']['memoryCapacity'] = query_filter(memory)
@@ -146,11 +156,17 @@ class HardwareManager(IdentifierMixin, object):
     def get_bare_metal_create_options(self):
         """ Retrieves the available options for creating a bare metal server.
 
-        The information for ordering bare metal instances comes from multiple
-        API calls. In order to make the process easier, this function will
-        make those calls and reformat the results into a dictionary that's
-        easier to manage. It's recommended that you cache these results with a
-        reasonable lifetime for performance reasons.
+        :returns: A dictionary of creation options. The categories to order are
+                  contained within the 'categories' key. See
+                  :func:`_parse_package_data` for detailed information.
+
+        .. note::
+
+           The information for ordering bare metal instances comes from
+           multiple API calls. In order to make the process easier, this
+           function will make those calls and reformat the results into a
+           dictionary that's easier to manage. It's recommended that you cache
+           these results with a reasonable lifetime for performance reasons.
         """
         hw_id = self._get_bare_metal_package_id()
 
@@ -163,10 +179,13 @@ class HardwareManager(IdentifierMixin, object):
         """ Retrieves a list of packages that are available for ordering
         dedicated servers.
 
-        Note - This currently returns a hard coded list until the API is
-        updated to allow filtering on packages to just those for ordering
-        servers.
+        :returns: A list of tuples of available dedicated server packages in
+                  the form (id, name, description)
         """
+
+        # Note - This currently returns a hard coded list until the API is
+        # updated to allow filtering on packages to just those for ordering
+        # servers.
         package_ids = [13, 15, 23, 25, 26, 27, 29, 32, 41, 42, 43, 44, 49, 51,
                        52, 53, 54, 55, 56, 57, 126, 140, 141, 142, 143, 144,
                        145, 146, 147, 148, 158]
@@ -188,11 +207,20 @@ class HardwareManager(IdentifierMixin, object):
         """ Retrieves the available options for creating a dedicated server in
         a specific chassis (based on package ID).
 
-        The information for ordering dedicated servers comes from multiple
-        API calls. In order to make the process easier, this function will
-        make those calls and reformat the results into a dictionary that's
-        easier to manage. It's recommended that you cache these results with a
-        reasonable lifetime for performance reasons.
+        :param int package_id: The package ID to retrieve the creation options
+                               for. This should come from
+                               :func:`get_available_dedicated_server_packages`.
+        :returns: A dictionary of creation options. The categories to order are
+                  contained within the 'categories' key. See
+                  :func:`_parse_package_data` for detailed information.
+
+        .. note::
+
+           The information for ordering dedicated servers comes from multiple
+           API calls. In order to make the process simpler, this function will
+           make those calls and reformat the results into a dictionary that's
+           easier to manage. It's recommended that you cache these results with
+           a reasonable lifetime for performance reasons.
         """
         return self._parse_package_data(package_id)
 
@@ -200,6 +228,8 @@ class HardwareManager(IdentifierMixin, object):
         """ Get details about a hardware device
 
         :param integer id: the hardware ID
+        :returns: A dictionary containing a large amount of information about
+                  the specified server.
 
         """
 
@@ -212,24 +242,28 @@ class HardwareManager(IdentifierMixin, object):
                 'domain',
                 'provisionDate',
                 'hardwareStatus',
-                'processorCoreAmount',
+                'processorPhysicalCoreAmount',
                 'memoryCapacity',
                 'notes',
+                'privateNetworkOnlyFlag',
                 'primaryBackendIpAddress',
                 'primaryIpAddress',
                 'userData',
-                'datacenter.name',
+                'datacenter',
                 'networkComponents[id, status, speed, maxSpeed, name,'
                 'ipmiMacAddress, ipmiIpAddress, macAddress, primaryIpAddress,'
                 'port, primarySubnet]',
                 'networkComponents.primarySubnet[id, netmask,'
                 'broadcastAddress, networkIdentifier, gateway]',
+                'hardwareChassis[id,name]',
                 'activeTransaction.id',
                 'operatingSystem.softwareLicense.'
                 'softwareDescription[manufacturer,name,version,referenceCode]',
                 'operatingSystem.passwords[username,password]',
                 'billingItem.recurringFee',
+                'hourlyBillingFlag',
                 'tagReferences[id,tag[name,id]]',
+                'networkVlans[id,vlanNumber,networkSpace]',
             ])
             kwargs['mask'] = "mask[%s]" % ','.join(items)
 
@@ -260,8 +294,8 @@ class HardwareManager(IdentifierMixin, object):
 
         :param int id: The ID of the server
         :param bool public: Flag to indicate which interface to change.
-                           True (default) means the public interface.
-                           False indicates the private interface.
+                            True (default) means the public interface.
+                            False indicates the private interface.
         :param int speed: The port speed to set.
         """
         if public:
@@ -272,20 +306,83 @@ class HardwareManager(IdentifierMixin, object):
         return func(speed, id=id)
 
     def place_order(self, **kwargs):
-        """ Places an order for a piece of hardware. See _generate_create_dict
-        for a list of available options.
+        """ Places an order for a piece of hardware. See
+        :func:`_generate_create_dict` for a list of available options.
+
+        .. warning::
+           Due to how the ordering structure currently works, all ordering
+           takes place using price IDs rather than quantities. See the
+           following sample for an example of using HardwareManager functions
+           for ordering a basic server.
+
+        ::
+
+           # client is assumed to be an initialized SoftLayer.API.Client object
+           mgr = HardwareManager(client)
+
+           # Package ID 32 corresponds to the 'Quad Processor, Quad Core Intel'
+           # package. This information can be obtained from the
+           # :func:`get_available_dedicated_server_packages` function.
+           options = mgr.get_dedicated_server_create_options(32)
+
+           # Review the contents of options to find the information that
+           # applies to your order. For the sake of this example, we assume
+           # that your selections are a series of item IDs for each category
+           # organized into a key-value dictionary.
+
+           # This contains selections for all required categories
+           selections = {
+               'server': 542, # Quad Processor Quad Core Intel 7310 - 1.60GHz
+               'pri_ip_addresses': 15, # 1 IP Address
+               'notification': 51, # Email and Ticket
+               'ram': 280, # 16 GB FB-DIMM Registered 533/667
+               'bandwidth': 173, # 5000 GB Bandwidth
+               'lockbox': 45, # 1 GB Lockbox
+               'monitoring': 49, # Host Ping
+               'disk0': 14, # 500GB SATA II (for the first disk)
+               'response': 52, # Automated Notification
+               'port_speed': 187, # 100 Mbps Public & Private Networks
+               'power_supply': 469, # Redundant Power Supplies
+               'disk_controller': 487, # Non-RAID
+               'vulnerability_scanner': 307, # Nessus
+               'vpn_management': 309, # Unlimited SSL VPN Users
+               'remote_management': 504, # Reboot / KVM over IP
+               'os': 4166, # Ubuntu Linux 12.04 LTS Precise Pangolin (64 bit)
+           }
+
+           args = {
+               'location': 'FIRST_AVAILABLE', # Pick the first available DC
+               'disks': [],
+           }
+
+           for cat, item_id in selections:
+               for item in options['categories'][cat]['items'].items():
+                   if item['id'] == item_id:
+                       if 'disk' not in cat or 'disk_controller' == cat:
+                           args[cat] = item['price_id']
+                       else:
+                           args['disks'].append(item['price_id'])
+
+           # You can call :func:`verify_order` here to test the order instead
+           # of actually placing it if you prefer.
+           result = mgr.place_order(**args)
+
         """
         create_options = self._generate_create_dict(**kwargs)
         return self.client['Product_Order'].placeOrder(create_options)
 
     def verify_order(self, **kwargs):
         """ Verifies an order for a piece of hardware without actually placing
-        it. See _generate_create_dict for a list of available options.
+        it. See :func:`_generate_create_dict` for a list of available options.
         """
         create_options = self._generate_create_dict(**kwargs)
         return self.client['Product_Order'].verifyOrder(create_options)
 
     def get_cancellation_reasons(self):
+        """
+        Returns a dictionary of valid cancellation reasons that can be used
+        when cancelling a dedicated server via :func:`cancel_hardware`.
+        """
         return {
             'unneeded': 'No longer needed',
             'closing': 'Business closing down',
@@ -302,15 +399,19 @@ class HardwareManager(IdentifierMixin, object):
     def _generate_create_dict(
             self, server=None, hostname=None, domain=None, hourly=False,
             location=None, os=None, disks=None, port_speed=None,
-            bare_metal=None, ram=None, package_id=None, disk_controller=None):
+            bare_metal=None, ram=None, package_id=None, disk_controller=None,
+            ssh_keys=None, public_vlan=None, private_vlan=None):
         """
         Translates a list of arguments into a dictionary necessary for creating
-        a server. NOTE - All items here must be price IDs, NOT quantities!
+        a server.
 
-        :param string server: The identification string for the server to
-                              order. This will either be the CPU/Memory
-                              combination ID for bare metal instances or the
-                              CPU model for dedicated servers.
+        .. warning::
+           All items here must be price IDs, NOT quantities!
+
+        :param int server: The identification string for the server to
+                           order. This will either be the CPU/Memory
+                           combination ID for bare metal instances or the
+                           CPU model for dedicated servers.
         :param string hostname: The hostname to use for the new server.
         :param string domain: The domain to use for the new server.
         :param bool hourly: Flag to indicate if this server should be billed
@@ -332,22 +433,39 @@ class HardwareManager(IdentifierMixin, object):
                                which can be obtained by calling
                                _get_bare_metal_package_id
         :param int disk_controller: The disk controller to use.
+        :param list ssh_keys: The SSH keys to add to the root user
+        :param int public_vlan: The ID of the public VLAN on which you want
+                                this server placed.
+        :param int private_vlan: The ID of the public VLAN on which you want
+                                 this server placed.
         """
         arguments = ['server', 'hostname', 'domain', 'location', 'os', 'disks',
                      'port_speed', 'bare_metal', 'ram', 'package_id',
                      'disk_controller', 'server_core', 'disk0']
 
+        hardware = {
+            'bareMetalInstanceFlag': bare_metal,
+            'hostname': hostname,
+            'domain': domain,
+        }
+
+        if public_vlan:
+            hardware['primaryNetworkComponent'] = {
+                "networkVlan": {"id": int(public_vlan)}}
+        if private_vlan:
+            hardware['primaryBackendNetworkComponent'] = {
+                "networkVlan": {"id": int(private_vlan)}}
+
         order = {
-            'hardware': [{
-                'bareMetalInstanceFlag': bare_metal,
-                'hostname': hostname,
-                'domain': domain,
-            }],
+            'hardware': [hardware],
             'location': location,
             'prices': [
             ],
         }
 
+        if ssh_keys:
+            order['sshKeys'] = [{'sshKeyIds': ssh_keys}]
+
         if bare_metal:
             order['packageId'] = self._get_bare_metal_package_id()
             order['prices'].append({'id': int(server)})
@@ -379,6 +497,12 @@ class HardwareManager(IdentifierMixin, object):
         required_fields = []
         for category, data in p_options['categories'].iteritems():
             if data.get('is_required') and category not in arguments:
+                if 'disk' in category:
+                    # This block makes sure that we can default unspecified
+                    # disks if the user hasn't specified enough.
+                    disk_count = int(category.replace('disk', ''))
+                    if len(disks) >= disk_count + 1:
+                        continue
                 required_fields.append(category)
 
         for category in required_fields:
@@ -411,7 +535,7 @@ class HardwareManager(IdentifierMixin, object):
         except socket.error:
             return []
 
-        # Find the CCI via ip address. First try public ip, then private
+        # Find the server via ip address. First try public ip, then private
         results = self.list_hardware(public_ip=ip, mask="id")
         if results:
             return [result['id'] for result in results]
@@ -420,7 +544,49 @@ class HardwareManager(IdentifierMixin, object):
         if results:
             return [result['id'] for result in results]
 
-    def _parse_package_data(self, id):
+    def _parse_package_data(self, package_id):
+        """
+        Parses data from the specified package into a consistent dictionary.
+
+        The data returned by the API varies significantly from one package
+        to another, which means that consuming it can make your program more
+        complicated than desired. This function will make all necessary API
+        calls for the specified package ID and build the results into a
+        consistently formatted dictionary like so:
+
+        result = {
+            'locations': [{'delivery_information': <string>,
+                           'keyname': <string>,
+                           'long_name': <string>}],
+            'categories': {
+                'category_code': {
+                    'sort': <int>,
+                    'step': <int>,
+                    'is_required': <bool>,
+                    'name': <string>,
+                    'group': <string>,
+                    'items': [
+                        {
+                            'id': <int>,
+                            'description': <string>,
+                            'sort': <int>,
+                            'price_id': <int>,
+                            'recurring_fee': <float>,
+                            'setup_fee': <float>,
+                            'hourly_recurring_fee': <float>,
+                            'one_time_fee': <float>,
+                            'labor_fee': <float>,
+                            'capacity': <float>,
+                        }
+                    ]
+                }
+            }
+        }
+
+        Your code can rely upon each of those elements always being present.
+        Each list will contain at least one entry as well, though most will
+        contain more than one.
+        """
         package = self.client['Product_Package']
 
         results = {
@@ -430,7 +596,7 @@ class HardwareManager(IdentifierMixin, object):
 
         # First pull the list of available locations. We do it with the
         # getObject() call so that we get access to the delivery time info.
-        object_data = package.getRegions(id=id)
+        object_data = package.getRegions(id=package_id)
 
         for loc in object_data:
             details = loc['location']['locationPackageDetails'][0]
@@ -443,7 +609,7 @@ class HardwareManager(IdentifierMixin, object):
 
         mask = 'mask[itemCategory[group]]'
 
-        for config in package.getConfiguration(id=id, mask=mask):
+        for config in package.getConfiguration(id=package_id, mask=mask):
             code = config['itemCategory']['categoryCode']
             group = NestedDict(config['itemCategory']) or {}
             category = {
@@ -458,28 +624,33 @@ class HardwareManager(IdentifierMixin, object):
             results['categories'][code] = category
 
         # Now pull in the available package item
-        for item in package.getItems(id=id, mask='mask[itemCategory]'):
-            category_code = item['itemCategory']['categoryCode']
+        for category in package.getCategories(id=package_id):
+            code = category['categoryCode']
+            items = []
 
-            if category_code not in results['categories']:
-                results['categories'][category_code] = {'name': category_code,
-                                                        'items': []}
-            results['categories'][category_code]['items'].append({
-                'id': item['id'],
-                'description': item['description'],
-                'prices': item['prices'],
-                'sort': item['prices'][0]['sort'],
-                'price_id': item['prices'][0]['id'],
-                'recurring_fee': float(item['prices'][0].get('recurringFee',
-                                                             0)),
-                'capacity': float(item.get('capacity', 0)),
-            })
+            for group in category['groups']:
+                for price in group['prices']:
+                    items.append({
+                        'id': price['itemId'],
+                        'description': price['item']['description'],
+                        'sort': price['sort'],
+                        'price_id': price['id'],
+                        'recurring_fee': price.get('recurringFee', 0),
+                        'setup_fee': price.get('setupFee', 0),
+                        'hourly_recurring_fee': price.get('hourlyRecurringFee',
+                                                          0),
+                        'one_time_fee': price.get('oneTimeFee', 0),
+                        'labor_fee': price.get('laborFee', 0),
+                        'capacity': float(price['item'].get('capacity', 0)),
+                    })
+
+            results['categories'][code]['items'] = items
 
         return results
 
     def edit(self, id, userdata=None, hostname=None, domain=None, notes=None):
-        """ Edit hostname, domain name, notes, and/or the
-            user data of the hardware
+        """ Edit hostname, domain name, notes, and/or the user data of the
+        hardware
 
         Parameters set to None will be ignored and not attempted to be updated.
 
@@ -512,15 +683,30 @@ class HardwareManager(IdentifierMixin, object):
 
 
 def get_default_value(package_options, category):
+    """ Returns the default price ID for the specified category.
+
+    This determination is made by parsing the items in the package_options
+    argument and finding the first item that has zero specified for every fee
+    field.
+
+    .. note::
+       If the category has multiple items with no fee, this will return the
+       first it finds and then short circuit. This may not match the default
+       value presented on the SoftLayer ordering portal. Additionally, this
+       method will return None if there are no free items in the category.
+
+    :returns: Returns the price ID of the first free item it finds or None
+              if there are no free items.
+    """
     if category not in package_options['categories']:
         return
 
     for item in package_options['categories'][category]['items']:
         if not any([
-            float(item['prices'][0].get('setupFee', 0)),
-            float(item['prices'][0].get('recurringFee', 0)),
-            float(item['prices'][0].get('hourlyRecurringFee', 0)),
-            float(item['prices'][0].get('oneTimeFee', 0)),
-            float(item['prices'][0].get('laborFee', 0)),
+            float(item.get('setupFee', 0)),
+            float(item.get('recurringFee', 0)),
+            float(item.get('hourlyRecurringFee', 0)),
+            float(item.get('oneTimeFee', 0)),
+            float(item.get('laborFee', 0)),
         ]):
             return item['price_id']
